// Copyright (c)2003, TopCoder, Inc. All rights reserved
// @author aksonov
// @version 1.0

using System;
using NUnit.Framework;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.Util.ConfigurationManager.FailureTests
{
    /// <summary>
    /// Failure tests for ConfigManager and related classes
    /// </summary>
    [TestFixture]
    public class ConfigManagerTests
    {
        /// <summary>
        /// ConfigManager instance
        /// </summary>
        private ConfigManager manager = null;

        private readonly string TEST_PREFIX = "../../test_files/";

		private readonly string TEST_INVALID_FILENAME = "not_here.ini";
        /// <summary>
        /// SetUp method creates ConfigManager instance
        /// </summary>
        [SetUp]
        protected void SetUp()
        {
            manager = ConfigManager.GetInstance();
            manager.LoadFile(TEST_PREFIX+"failure_tests.xml");
        }

        /// <summary>
        /// Tests LoadFile(string) method with null argument
        /// </summary>
        [Test]
        [ExpectedException(typeof(System.NullReferenceException))]
        public void LoadFileStringNull()
        {
            manager.LoadFile(null);
        }

        /// <summary>
        /// Tests ReloadFile(string) method with null argument
        /// </summary>
        [Test]
        [ExpectedException(typeof(System.NullReferenceException))]
        public void ReloadFileStringNull()
        {
            manager.ReloadFile(null);
        }

        /// <summary>
        /// Tests LoadFile(string) method with unexist configuration filename (IOException should be thrown)
        /// </summary>
        [Test]
        public void LoadFileStringInvalidFilename()
        {
			try
			{
				manager.LoadFile(TEST_INVALID_FILENAME);
				Assertion.Fail();
			}
			catch (System.IO.IOException)
			{
			}
			catch
			{
				Assertion.Fail();
			}
        }

        /// <summary>
        /// Tests ReloadFile(string) method with unexist configuration filename (IOException should be thrown)
        /// </summary>
        [Test]
        public void ReloadFileStringInvalidFilename()
        {
			try
			{
				manager.ReloadFile(TEST_INVALID_FILENAME);
				Assertion.Fail();
			}
			catch (System.IO.IOException)
			{
			}
			catch
			{
				Assertion.Fail();
			}
		}

        /// <summary>
        /// Tests LoadFile(string,string) method with unexist configuration filename (IOException should be thrown)
        /// </summary>
        [Test]
        public void LoadFileStringStringInvalidFilename()
        {
			try
			{
				manager.LoadFile(TEST_INVALID_FILENAME, "test");
				Assertion.Fail();
			}
			catch (System.IO.IOException)
			{
			}
			catch
			{
				Assertion.Fail();
			}
		}

        /// <summary>
        /// Tests LoadFile(string,string) method with first null argument
        /// </summary>
        [Test]
		[ExpectedException(typeof(System.NullReferenceException))]
        public void LoadFileStringStringFirstNull()
        {
			manager.LoadFile(null, "test");
		}

        /// <summary>
        /// Tests LoadFile(string,string) method with second null argument
        /// </summary>
        [Test]
        public void LoadFileStringStringSecondNull()
        {
            try
			{
				manager.LoadFile(TEST_INVALID_FILENAME, (string)null);
				Assertion.Fail();
			}
			catch (System.IO.IOException)
			{
			}
			catch
			{
				Assertion.Fail();
			}
		}

        /// <summary>
        /// Tests LoadFile(string,ErrorCallback) method with first null argument
        /// </summary>
        [Test]
		[ExpectedException(typeof(System.NullReferenceException))]
		public void LoadFileStringErrorCallbackFirstNull()
        {
			manager.LoadFile(null, new DefaultErrorCallback());
        }

        /// <summary>
        /// Tests LoadFile(string,ErrorCallback) method with unexisted file
        /// </summary>
        [Test]
        public void LoadFileStringErrorCallbackFirstInvalid()
        {
            try
			{
				manager.LoadFile(TEST_INVALID_FILENAME, new DefaultErrorCallback());
				Assertion.Fail();
			}
			catch (System.IO.IOException)
			{
			}
			catch
			{
				Assertion.Fail();
			}
		}

        /// <summary>
        /// Tests LoadFile(string,ErrorCallback) method with second null argument
        /// </summary>
        [Test]
        public void LoadFileStringErrorCallbackSecondNull()
        {
            try
			{
				manager.LoadFile(TEST_INVALID_FILENAME, (ErrorCallback)null);
				Assertion.Fail();
			}
			catch (System.IO.IOException)
			{
			}
			catch
			{
				Assertion.Fail();
			}
		}

        /// <summary>
        /// Tests LoadFile(string,string,ErrorCallback) method with first null argument
        /// </summary>
        [Test]
		[ExpectedException(typeof(System.NullReferenceException))]
		public void LoadFileStringStringErrorCallbackFirstNull()
        {
			manager.LoadFile(null, "test", new DefaultErrorCallback());
		}

        /// <summary>
        /// Tests LoadFile(string,string,ErrorCallback) method with second null argument
        /// </summary>
        [Test]
        public void LoadFileStringStringErrorCallbackSecondNull()
        {
            try
			{
				manager.LoadFile(TEST_INVALID_FILENAME, (string)null, new DefaultErrorCallback());
				Assertion.Fail();
			}
			catch (System.IO.IOException)
			{
			}
			catch
			{
				Assertion.Fail();
			}
		}

        /// <summary>
        /// Tests LoadFile(string,string,ErrorCallback) method with third null argument
        /// </summary>
        [Test]
        public void LoadFileStringStringErrorCallbackThirdNull()
        {
            try
			{
				manager.LoadFile(TEST_INVALID_FILENAME, "test", null);
				Assertion.Fail();
			}
			catch (System.IO.IOException)
			{
			}
			catch
			{
				Assertion.Fail();
			}
		}

        /// <summary>
        /// Tests LoadFile(string,string,ErrorCallback) method with unexisted file
        /// </summary>
        [Test]
        public void LoadFileStringStringErrorCallbackFirstInvalid()
        {
            try
			{
				manager.LoadFile(TEST_INVALID_FILENAME, "test", new DefaultErrorCallback());
				Assertion.Fail();
			}
			catch (System.IO.IOException)
			{
			}
			catch
			{
				Assertion.Fail();
			}
		}

        /// <summary>
        /// Tests LoadFile(string) method for NamespaceClashException
        /// </summary>
        [Test]
		[ExpectedException(typeof(NamespaceClashException))]
		public void LoadFileStringNamespaceClashException()
        {
			manager.LoadFile(TEST_PREFIX+"failure_tests.xml");
		}

        /// <summary>
        /// Tests LoadFile(string, string) method for NamespaceClashException
        /// </summary>
        [Test]
        [ExpectedException(typeof(NamespaceClashException))]
        public void LoadFileStringStringNamespaceClashException()
        {
			manager.LoadFile(TEST_PREFIX+"failure_tests.xml", "FailureTests");
		}

        /// <summary>
        /// Tests LoadFile(string, string, ErrorCallBack) method for NamespaceClashException
        /// </summary>
        [Test]
		[ExpectedException(typeof(NamespaceClashException))]
		public void LoadFileStringStringErrorCallbackNamespaceClashException()
        {
			manager.LoadFile(TEST_PREFIX+"failure_tests.xml", "FailureTests", new DefaultErrorCallback());
		}

        /// <summary>
        /// Tests LoadFile(string, ErrorCallBack) method for NamespaceClashException
        /// </summary>
        [Test]
		[ExpectedException(typeof(NamespaceClashException))]
		public void LoadFileStringErrorCallbackNamespaceClashException()
        {
			manager.LoadFile(TEST_PREFIX+"failure_tests.xml", new DefaultErrorCallback());
		}

        /// <summary>
        /// Tests LoadFile(string) method for PropertyClashException
        /// </summary>
        [Test]
		[ExpectedException(typeof(PropertyClashException))]
		public void LoadFileStringPropertyClashException()
        {
			manager.LoadFile(TEST_PREFIX+"failure_tests2.xml");
		}

        /// <summary>
        /// Tests ReloadFile(string) method for PropertyClashException
        /// </summary>
        [Test]
		[ExpectedException(typeof(PropertyClashException))]
		public void ReloadFileStringPropertyClashException()
        {
			manager.ReloadFile(TEST_PREFIX+"failure_tests2.xml");
		}

        /// <summary>
        /// Tests LoadFile(string, string) method for PropertyClashException
        /// </summary>
        [Test]
		[ExpectedException(typeof(PropertyClashException))]
		public void LoadFileStringStringPropertyClashException()
        {
			manager.LoadFile(TEST_PREFIX+"failure_tests2.xml", "FailureTests");
		}

        /// <summary>
        /// Tests LoadFile(string, string, ErrorCallBack) method for PropertyClashException
        /// </summary>
        [Test]
		[ExpectedException(typeof(PropertyClashException))]
		public void LoadFileStringStringErrorCallbackPropertyClashException()
        {
			manager.LoadFile(TEST_PREFIX+"failure_tests2.xml", "FailureTests", new DefaultErrorCallback());
		}

        /// <summary>
        /// Tests LoadFile(string, ErrorCallBack) method for PropertyClashException
        /// </summary>
        [Test]
		[ExpectedException(typeof(PropertyClashException))]
		public void LoadFileStringErrorCallbackPropertyClashException()
        {
			manager.LoadFile(TEST_PREFIX+"failure_tests2.xml", new DefaultErrorCallback());
		}

        
        /// <summary>
        /// Tests LoadFile(string) method for InvalidConfigFileException
        /// </summary>
        [Test]
		[ExpectedException(typeof(InvalidConfigFileException))]
		public void LoadFileStringInvalidConfigFileException()
        {
			manager.LoadFile(TEST_PREFIX+"failure_tests3.xml");
		}

        /// <summary>
        /// Tests ReloadFile(string) method for InvalidConfigFileException
        /// </summary>
        [Test]
		[ExpectedException(typeof(InvalidConfigFileException))]
		public void ReloadFileStringInvalidConfigFileException()
        {
			manager.ReloadFile(TEST_PREFIX+"failure_tests3.xml");
		}

        /// <summary>
        /// Tests LoadFile(string, string) method for InvalidConfigFileException
        /// </summary>
        [Test]
		[ExpectedException(typeof(InvalidConfigFileException))]
		public void LoadFileStringStringInvalidConfigFileException()
        {
			manager.LoadFile(TEST_PREFIX+"failure_tests3.xml", "FailureTests");
		}

        /// <summary>
        /// Tests LoadFile(string, string, ErrorCallBack) method for InvalidConfigFileException
        /// </summary>
        [Test]
		[ExpectedException(typeof(InvalidConfigFileException))]
		public void LoadFileStringStringErrorCallbackInvalidConfigFileException()
        {
			manager.LoadFile(TEST_PREFIX+"failure_tests3.xml", "FailureTests", new DefaultErrorCallback());
		}

        /// <summary>
        /// Tests LoadFile(string, ErrorCallBack) method for InvalidConfigFileException
        /// </summary>
        [Test]
		[ExpectedException(typeof(InvalidConfigFileException))]
		public void LoadFileStringErrorCallbackInvalidConfigFileException()
        {
			manager.LoadFile(TEST_PREFIX+"failure_tests3.xml", new DefaultErrorCallback());
		}
        
        

        /// <summary>
        /// Tests LoadFile(string) method for InvalidConfigFileException
        /// </summary>
        [Test]
		[ExpectedException(typeof(InvalidConfigFileException))]
		public void LoadFileStringInvalidIniConfigFileException()
        {
			manager.LoadFile(TEST_PREFIX+"failure_tests3.ini");
		}

        /// <summary>
        /// Tests ReloadFile(string) method for InvalidConfigFileException
        /// </summary>
        [Test]
		[ExpectedException(typeof(InvalidConfigFileException))]
		public void ReloadFileStringInvalidIniConfigFileException()
        {
			manager.ReloadFile(TEST_PREFIX+"failure_tests3.ini");
		}

        /// <summary>
        /// Tests LoadFile(string, string) method for InvalidConfigFileException
        /// </summary>
        [Test]
		[ExpectedException(typeof(InvalidConfigFileException))]
		public void LoadFileStringStringInvalidIniConfigFileException()
        {
			manager.LoadFile(TEST_PREFIX+"failure_tests3.ini", "FailureTests");
		}

        /// <summary>
        /// Tests LoadFile(string, string, ErrorCallBack) method for InvalidConfigFileException
        /// </summary>
        [Test]
		[ExpectedException(typeof(InvalidConfigFileException))]
		public void LoadFileStringStringErrorCallbackInvalidIniConfigFileException()
        {
			manager.LoadFile(TEST_PREFIX+"failure_tests3.ini", "FailureTests", new DefaultErrorCallback());
		}

        /// <summary>
        /// Tests LoadFile(string, ErrorCallBack) method for InvalidConfigFileException
        /// </summary>
        [Test]
		[ExpectedException(typeof(InvalidConfigFileException))]
		public void LoadFileStringErrorCallbackInvalidIniConfigFileException()
        {
			manager.LoadFile(TEST_PREFIX+"failure_tests3.ini", new DefaultErrorCallback());
		}
        

        /// <summary>
        /// Tests LoadFile(string) method for UnknownFormatException
        /// </summary>
        [Test]
		[ExpectedException(typeof(UnknownFormatException))]
		public void LoadFileStringUnknownFormatException()
        {
			manager.LoadFile(TEST_PREFIX+"failure_tests4.tmp");
		}

        /// <summary>
        /// Tests ReloadFile(string) method for UnknownFormatException
        /// </summary>
        [Test]
		[ExpectedException(typeof(UnknownFormatException))]
		public void ReloadFileStringUnknownFormatException()
        {
			manager.ReloadFile(TEST_PREFIX+"failure_tests4.tmp");
		}

        /// <summary>
        /// Tests LoadFile(string, string) method for UnknownFormatException
        /// </summary>
        [Test]
		[ExpectedException(typeof(UnknownFormatException))]
		public void LoadFileStringStringUnknownFormatException()
        {
			manager.LoadFile(TEST_PREFIX+"failure_tests4.tmp", "FailureTests");
		}

        /// <summary>
        /// Tests LoadFile(string, string, ErrorCallBack) method for UnknownFormatException
        /// </summary>
        [Test]
		[ExpectedException(typeof(UnknownFormatException))]
		public void LoadFileStringStringErrorCallbackUnknownFormatException()
        {
			manager.LoadFile(TEST_PREFIX+"failure_tests4.tmp", "FailureTests", new DefaultErrorCallback());
		}

        /// <summary>
        /// Tests LoadFile(string, ErrorCallBack) method for UnknownFormatException
        /// </summary>
        [Test]
		[ExpectedException(typeof(UnknownFormatException))]
		public void LoadFileStringErrorCallbackUnknownFormatException()
        {
			manager.LoadFile(TEST_PREFIX+"failure_tests4.tmp", new DefaultErrorCallback());
		}

        /// <summary>
        /// Tests ReloadFile(string) method for MergeException  (failure_tests5.xml has the same namespace as failure_tests.xml)
        /// </summary>
        [Test]
		[ExpectedException(typeof(MergeException))]
		public void ReloadFileStringMergeException()
        {
			manager.LoadFile(TEST_PREFIX+"failure_tests5.xml", new MyErrorCallback());
			manager.ReloadFile(TEST_PREFIX+"failure_tests5.xml");
			manager.RemoveFile(TEST_PREFIX+"failure_tests5.xml");
        }

        /// <summary>
        /// Tests SaveFile(string) method for MergeException (failure_tests5.xml has the same namespace as failure_tests.xml)
        /// </summary>
        [Test]
		[ExpectedException(typeof(MergeException))]
		public void SaveFileStringMergeException()
        {
			manager.LoadFile(TEST_PREFIX+"failure_tests5.xml", new MyErrorCallback());
			manager.SaveFileTo(TEST_PREFIX+"failure_tests5.xml",TEST_PREFIX+"failure_tests5_.xml");
			System.IO.File.Delete(TEST_PREFIX+"failure_tests5_.xml");
			manager.RemoveFile(TEST_PREFIX+"failure_tests5.xml");
		}

        /// <summary>
        /// tear down method
        /// </summary>
        [TearDown]
        protected void TearDown()
        {
			manager.Clear();
		}
    }
}
