// Copyright (c)2005, TopCoder, Inc. All rights reserved
// @author TCSDEVELOPER

using System;
using System.Threading;
using NUnit.Framework;

namespace TopCoder.Util.ConfigurationManager.StressTests
{
	/// <summary>
	/// New concurency tests for version 1.1 of ConfigManager.
	/// </summary>
	[TestFixture]
    public class NewStressConcurencyTests {
        /// <summary>
        /// The prefix for all filenames.
        /// </summary>
        private const string PREFIX = @"..\..\test_files\stress\";

        /// <summary>
        /// how many times we want to run test
        /// </summary>
        private int cnt;

        /// <summary>
        /// instance of StressTestsUtils
        /// </summary>
        private StressTestsUtils tu = null;

        /// <summary>
        /// namespace used in testing
        /// </summary>
        private string nameSpace = "nsValue";

        /// <summary>
        /// The configuration manager instance.
        /// </summary>
        private ConfigManager cm = null;

        /// <summary>
        /// exception saved here to be able to throw it later
        /// </summary>
        private Exception exc = null;
        
        /// <summary>
        /// Set up.
        /// </summary>
        [SetUp]
        public void SetUp() {
            cm = ConfigManager.GetInstance();
            tu = new StressTestsUtils();
            tu.units = " config per sec";
        }

        /// <summary>
        /// Running with saving simple value to property aaa
        /// </summary>
        private void RunSaveValue() {
            try {
                for (int i = 0; i< cnt; i++) {
                    cm.SetValue(nameSpace, "aaa", "bbbb");
                }
                string val = cm.GetValue("nsValue", "aaa");
                Assert.AreEqual("bbbb", val, "string value");
            } catch (Exception e) {
                exc = e;
            }
        }

        /// <summary>
        /// testing with saving simple property
        /// </summary>
        [Test]
        public void TestSaveValue() {
                    cm.Clear();
                    cm.LoadFile(PREFIX + "stressValue.xml");
            ThreadStart tr = new ThreadStart(RunSaveValue);
            tu.RunInThreads("ConfigManager", "save value", 100, 10, tr, out cnt);            
            if (exc != null) {
                throw exc;
            }
        }

        /// <summary>
        /// running with concurrently updating array property from 
        /// many threads. This test tries to catch thread safety errors
        /// that can happen in updating config manager namespace.
        /// </summary>
        private void RunConcurrentCheckUpdate() {
            try {
                for (int i = 0; i< cnt; i++) {
                    string []values = new string[]{"1","3","5","67","2"};
                    cm.SetValues(nameSpace, "aaa", values);
                    string []vals = cm.GetValues(nameSpace, "aaa");
                    AreEqual(values, vals, "We read the same values");
                }
            } catch (Exception e) {
                exc = e;
            }
        }

        /// <summary>
        /// Verifies two arrays are equivalent.
        /// </summary>
        /// <param name="a">the first array</param>
        /// <param name="b">the second array</param>
        /// <param name="message">the error message</param>
        private void AreEqual(Array a, Array b, string message)
        {
            Assert.AreEqual(a.Length, b.Length, message);
            for (int i = 0; i < a.Length; ++i)
            {
                Assert.AreEqual(a.GetValue(i), b.GetValue(i), message);
            }
        }

        /// <summary>
        /// test with repeatedly calling RunConcurrentCheckUpdate in 10 threads
        /// </summary>
        [Test]
        public void TestConcurrentCheckUpdate() {
                    cm.Clear();
                    cm.LoadFile(PREFIX + "stressValue.xml");
            ThreadStart tr = new ThreadStart(RunConcurrentCheckUpdate);
            tu.RunInThreads("ConfigManager", "ConcurrentCheckUpdate", 100, 10, tr, out cnt);            
            if (exc != null) {
                throw exc;
            }
        }

        /// <summary>
        /// running with getting value of property
        /// </summary>
        private void RunGetValue() {
            try {
                for (int i = 0; i< cnt; i++) {
                    cm.GetValue("nsValue", "aaa");
                }
                string val = cm.GetValue("nsValue", "aaa");
                Assert.AreEqual("ccccc", val, "string value");
            } catch (Exception e) {
                exc = e;
            }
        }

        /// <summary>
        /// trepeatedly run RunGetValue() method in 10 threads.
        /// </summary>
        [Test]
        public void TestGetValue() {
                    cm.Clear();
                    cm.LoadFile(PREFIX + "stressValue.xml");
            ThreadStart tr = new ThreadStart(RunGetValue);
            tu.RunInThreads("ConfigManager", "Get value", 100, 10, tr, out cnt);            
            if (exc != null) {
                throw exc;
            }
        }

    }
}
